#! /usr/bin/env /proj/axaf/bin/perl

# --8<--8<--8<--8<--
#
# Copyright (C) 2011 Smithsonian Astrophysical Observatory
#
# This file is part of analycent
#
# analycent is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -->8-->8-->8-->8--


use strict;
use warnings;

use File::Basename;
use Getopt::Long;
use Getopt::ArgvFile;

use Data::Dumper;

use PDL;
use PDL::Graphics::Limits qw/ limits /;
use PGPLOT::Device::PGWin;

use Chandra::Tools::Common qw( good_time_mask );

use CXC::Algorithm::Center2D qw[ center2D_events ];
use PDL::Algorithm::Center qw[ sigma_clip ];

use RDB;

use constant XCOLOR => 'red';
use constant YCOLOR => 'blue';


our $VERSION = '1.17.3';

our $prog = basename( $0 );
our %opt;

our %PlotOpts = (
    Font     => 'Normal',
    HardFont => 'Normal',
    HardLW   => 1,
    HardCH   => 1,
    CharSize => 1,
    Unit     => 'in',
    Color    => 'black'
);

our %PlotEnvOpts = ( AxisColour => 'black' );



eval { main() };
if ( $@ ) {
    print STDERR "# $prog: ", $@;
    exit( 1 );
}

exit( 0 );


sub main {
    parse_args();

    help( 1 ) if $opt{help};
    help( 2 ) if $opt{usage};

    do { print "$prog $VERSION\n"; return }
      if $opt{version};

    print "Reading events: $opt{input}\n"
      if $opt{verbose};

    my ( $hdr, $x, $y, $time ) = get_data();

    my $cent
      = $opt{force_ctr}
      ? { center => pdl( $opt{xc}, $opt{yc} ) }
      : center2D_events(
        coords  => [ $x, $y ],
        center  => [ $opt{xc}, $opt{yc} ],
        iterlim => $opt{iter},
        log     => !!$opt{verbose},
        clip    => $opt{iclip},
        nsigma  => $opt{nsigma},
        dtol    => $opt{dtol},
        pixcent => $opt{pixcent},
      );


    my ( $xcent, $ycent ) = $cent->center->list;

    print "Initial center: ($xcent, $ycent)\n"
      if $opt{verbose};

    if ( exists $opt{binw} ) {
        $opt{nbins} = int( ( $time->max - $time->min ) / $opt{binw} );
        $opt{nbins}++
          if $time->min + $opt{nbins} * $opt{binw} < $time->max;
    }
    else {
        $opt{binw} = ( $time->max - $time->min ) / $opt{nbins};
    }

    # generate time bins. don't use PDL hist function, as it truncates
    # the number of bins.

    my $tmin        = $time->min + sequence( $opt{nbins} ) * $opt{binw};
    my $timin       = vsearch( $tmin, $time );
    my $tmid        = $tmin + $opt{binw} / 2;
    my $time_cnts   = histogram( $time, $opt{binw}, $time->min, $opt{nbins} );
    my $clip_radius = zeroes( double, $opt{nbins} );

    my $binmask = zeroes( byte, $opt{nbins} );

    my $inclip = zeroes( long, $x->nelem );

    my %data;

    my @header = ( qw( time n r) );

    for my $coord ( qw( x y ) ) {
        for my $quant ( qw( cent ave rms median fq lq ) ) {
            $data{$coord}{$quant} = zeroes( double, $opt{nbins} );
            push @header, $coord . '_' . $quant;
        }

        $data{$coord}{n} = zeroes( long, $opt{nbins} );
        push @header, $coord . '_' . 'n';
    }

    my ( $x_last, $y_last ) = ( $xcent, $ycent );

    my @idx;
    # now, loop through time bins, centering on each bin and doing our
    # statistical magic
    print "Analyzing time bins\n"
      if $opt{verbose};
    for my $bin ( 0 .. $timin->nelem - 1 ) {
        my $imin = $timin->at( $bin );
        my $imax
          = $bin == $timin->nelem - 1
          ? $time->nelem - 1
          : $timin->at( $bin + 1 );

        my $idxs = $idx[$bin] = [ $imin, $imax ];
        my @times = ( $time->at( $imin ), $time->at( $imax ) );

        printf "bin % 3d: Time: [%14.8g, %14.8g]\n", $bin, @times
          if $opt{verbose} > 1;

        next if $imin == $imax;

        my $xb = $x->mslice( $idxs );
        my $yb = $y->mslice( $idxs );

        # reject points outside clipping radius

        my $center
          = center_data( $xb, $yb, $x_last, $y_last, $opt{verbose} > 2 );

        next if $center->error || 0 == $center->nelem;

        ( $x_last, $y_last ) = $center->center->list;

        my $clip_r = $opt{clip_r} || $opt{clip_sigma} * $center->sigma;
        $clip_radius->set( $bin, $clip_r );

        printf "bin % 3d: Time: [%14.8g, %14.8g]\tCenter: ( %f, %f )\tradius: %f\n",
          $bin, @times, $x_last, $y_last, $clip_r
          if $opt{verbose} > 1;

        my $clip = $inclip->mslice( $idxs );
        $clip .= ( $xb - $x_last )**2 + ( $yb - $y_last )**2 <= ( $clip_r )**2;

        ( $xb, $yb ) = where( $xb, $yb, $clip );

        $data{x}{cent}->set( $bin, $x_last );
        $data{y}{cent}->set( $bin, $y_last );

        for ( [ x => $xb ], [ 'y' => $yb ] ) {
            my ( $axis, $piddle ) = @{$_};

            my ( $ave, $rms, $median ) = $piddle->stats;


            $data{$axis}{ave}->set( $bin, $ave );
            $data{$axis}{rms}->set( $bin, $rms );
            $data{$axis}{median}->set( $bin, $median );

            $data{$axis}{fq}->set( $bin, $piddle->pct( 0.25 ) );
            $data{$axis}{lq}->set( $bin, $piddle->pct( 0.75 ) );

            $data{$axis}{n}->set( $bin, $piddle->nelem );

        }

        $binmask->set( $bin, 1 );
    }

    $data{x}{cent} -= $xcent;
    $data{y}{cent} -= $ycent;

    # get rid of bins where the data were obviously really not close to the
    # center
    print "Determining bad bins\n"
      if $opt{verbose};

    $cent = sigma_clip(
        coords     => $clip_radius,
        log        => $opt{verbose} > 1,
        save_mask  => 1,
        nsigma     => 3,
        mask       => $binmask,
        dtol       => 0.01 * $clip_radius->avg,
    );
    die( $cent->error ) if $cent->error;

    $binmask = $cent->mask;

    write_data( $opt{tag}, $tmin, $tmin + $opt{binw}, $binmask, \%data );

    print "Rejecting bins\n"
      if $opt{verbose};

    for my $coord ( qw( x y ) ) {
        $data{$coord}{$_} = $data{$coord}{$_}->where( $binmask )
          foreach keys %{ $data{$coord} };
    }

    $tmid = $tmid->where( $binmask );

    # zap the overall clipping list.
    $binmask->inplace->not;
    my @zapped = $binmask->which->list;
    $inclip->mslice( $idx[$_] ) .= 0 foreach @zapped;

    print "Rejecting bins: ", join( ', ', @zapped ), "\n"
      if $opt{verbose} && @zapped;

    ( $x, $y, $time ) = where( $x, $y, $time, $inclip );

    ( my $object = $hdr->value('OBJECT') || '' ) =~ s/'\s*(.*\S)\s*'/$1/;
    ( my $obsid  = $hdr->value('OBS_ID') || '' ) =~ s/'\s*(.*\S)\s*'/$1/;
    ( my $date   = $hdr->value('DATE')   || '' ) =~ s/'\s*(.*\S)\s*'/$1/;

    my @title;
    push @title, $object        if $object;
    push @title, "OBSID $obsid" if $obsid;
    push @title, "[$date]"      if $date;

    my $title = join( ' ', @title );

    $opt{title} = $opt{title} ? $title : '';

    my $win;
    my $device;

    #===================================

    my $pgwin = PGPLOT::Device::PGWin->new( {
        Device  => $opt{device},
        WinOpts => \%PlotOpts,
    } );


    {
        my $win = $pgwin->next( "$opt{tag}_events.ps/cps" );

        my $dx = $x - $xcent;
        my $dy = $y - $ycent;

        my @limits = limits( [ $time, $dx ], [ $time, $dy ] );

        $win->env( @limits,
            { %PlotEnvOpts, PlotPosition => [ 0.1, 0.9, 0.5, 0.9 ], } );

        $win->points(
            $time,
            $x - $xcent,
            { symbol => -1, HardLW => 1, LineWidth => 1, color => XCOLOR } );
        $win->label_axes( undef, "X $opt{unit}", $title );

        $win->env( @limits,
            { %PlotEnvOpts, PlotPosition => [ 0.1, 0.9, 0.1, 0.5 ] } );

        $win->points(
            $time,
            $y - $ycent,
            { symbol => -1, HardLW => 1, LineWidth => 1, color => YCOLOR } );
        $win->label_axes( 'Time [s]', "Y $opt{unit}", undef );

        $win->env( 0, 1, 0, 1, { Axis => undef } );
        $win->release;
    }

    #===================================

    plot_points( $pgwin, 'dist', 'Distance to Center',
        $data{x}{cent}, $tmid, $data{'y'}{cent}, $tmid, );

    plot_motion( $pgwin, $data{x}{cent}, $data{'y'}{cent} );

    plot_points( $pgwin, 'rms', 'RMS', $data{x}{rms}, $tmid,
        $data{y}{rms}, $tmid, );

    plot_n( $pgwin, $data{x}{n}, $tmid );

    plot_hist( $pgwin, 'dist', 'Distance', $opt{hist_d_width}, $data{x}{cent},
        $data{y}{cent}, @{ $opt{hist_d_lims} } );

    plot_hist( $pgwin, 'rms', 'RMS', $opt{hist_r_width}, $data{x}{rms},
        $data{y}{rms}, @{ $opt{hist_r_lims} } );


    {

        my $x_mq = $data{x}{lq} - $data{x}{fq};
        my $y_mq = $data{y}{lq} - $data{y}{fq};

        plot_points( $pgwin, 'mq', 'Last - First Quartiles',
            $x_mq, $tmid, $y_mq, $tmid );

        plot_hist( $pgwin, 'mq', 'Last - First Quartiles',
            $opt{hist_p_width}, $x_mq, $y_mq, @{ $opt{hist_p_lims} } );
    }

}

sub plot_points {
    my ( $pgwin, $tag, $label, $x, $xt, $y, $yt, $lx, $ly ) = @_;

    $lx = 0.05 unless defined $lx;
    $ly = 0.9  unless defined $ly;

    my $win = $pgwin->next( "$opt{tag}_pts_$tag" );

    my @minmax = limits( [ $xt, $x ], [ $yt, $y ] );

    $minmax[3] *= 1.1;
    $win->env( @minmax, \%PlotEnvOpts );

    $win->points( $xt, $x, { symbol => 'plus', color => XCOLOR } );

    $win->hold;

    $win->points( $yt, $y, { symbol => 'circle', color => YCOLOR } );

    $win->label_axes( 'Time', "$label $opt{unit}", $opt{title} );

    $win->legend(
        [ 'X', 'Y' ],
        $minmax[0] + $lx * ( $minmax[1] - $minmax[0] ),
        $minmax[2] + $ly * ( $minmax[3] - $minmax[2] ),
        {
            Symbol => [qw( plus circle )],
            Color  => [ XCOLOR, YCOLOR ],
            Bg     => 'white'
        } );

    $win->release;
}

sub plot_motion {
    my ( $pgwin, $x, $y ) = @_;

    my $tag   = 'motion';
    my $label = 'Motion';

    my $win = $pgwin->next( "$opt{tag}_pts_$tag" );

    my @minmax = limits( [ $x, $y ] );

    $win->env( @minmax, \%PlotEnvOpts );

    # slow!
    $win->arrow(
        $x->at( $_ ),
        $y->at( $_ ),
        $x->at( $_ + 1 ),
        $y->at( $_ + 1 ) ) foreach 0 .. $x->nelem - 2;

    $win->label_axes( "X $opt{unit}", "Y $opt{unit}", $opt{title} );
}

sub plot_n {
    my ( $pgwin, $n, $t ) = @_;

    my $tag   = 'n';
    my $label = 'N';

    my $win = $pgwin->next( "$opt{tag}_pts_$tag" );

    my @minmax = limits( [ $t, $n ] );

    $win->env( @minmax, \%PlotEnvOpts );

    $win->points( $t, $n, { symbol => 'plus' } );

    $win->label_axes( 'Time', "$label", $opt{title} );
}

sub plot_hist {
    my ( $pgwin, $tag, $label, $binw, $x, $y, $xmin, $xmax, $ymin, $ymax ) = @_;

    my $win = $pgwin->next( "$opt{tag}_hist_$tag" );

    my @xlim = limits( $x, $y, { Clean => 'RangeFrac' } );

    $xlim[0] = $xmin if defined $xmin;
    $xlim[1] = $xmax if defined $xmax;

    $binw //= ( $xlim[1] - $xlim[0] ) / 25;

    my ( $xvals, $xh ) = hist( $x, @xlim, $binw );
    my ( $yvals, $yh ) = hist( $y, @xlim, $binw );

    my @ylim = limits( [$xh], [$yh], { Clean => 'RangeFrac' } );

    $ylim[0] = $ymin if defined $ymin;
    $ylim[1] = $ymax if defined $ymax;

    $win->env( @xlim, 0, $ylim[1], \%PlotEnvOpts );

    $win->bin( $xvals, $xh, { linestyle => 3, color => XCOLOR } );
    $win->bin( $yvals, $yh, { linestyle => 1, color => YCOLOR } );

    $win->label_axes( "$label $opt{unit}", 'N', $opt{title} );

    $win->legend(
        [ 'X', 'Y' ],
        $xlim[0] + 0.7 * ( $xlim[1] - $xlim[0] ),
        $ylim[1] - 0.1 * ( $ylim[1] - $ylim[0] ),
        {
            linestyle => [ 3,      1 ],
            color     => [ XCOLOR, YCOLOR ],
        } );
}

sub get_data {

    require Astro::FITS::CFITSIO::Simple;

    my @cols = @opt{ 'xcol', 'ycol', 'tcol' };

    my ( $hdr, $x, $y, $time ) = Astro::FITS::CFITSIO::Simple::rdfits(
        $opt{input},
        @cols,
        {
            extname => $opt{extname},
            defined $opt{filter} ? ( rfilter => $opt{filter} ) : (),
            rethdr => 1,
            dtypes => { map { $_ => double } @cols },
            status => $opt{verbose} > 1,
        },

    ) or die( "error reading $opt{input}\n" );


    if ( defined $opt{gti} ) {
        print "Reading gti: $opt{gti}\n"
          if $opt{verbose};

        my ( $tstart, $tstop ) = rdfits(
            $opt{gti},
            qw( start stop ),
            {
                dtypes => {
                    start => double(),
                    stop  => double() } } );

        print "Applying gti\n"
          if $opt{verbose};

        # assume GTI is in order.  Is this wise?
        ( $x, $y, $time )
          = map { $_->sever }
          where( $x, $y, $time, good_time_mask( $time, $tstart, $tstop ) );
    }

    if ( $opt{scale} != 1 ) {
        $x *= $opt{scale};
        $y *= $opt{scale};
    }

    return ( $hdr, $x, $y, $time );
}


sub write_data {
    my ( $tag, $tmin, $tmax, $mask, $data ) = @_;

    my $file = "${tag}_stats.rdb";

    print "Writing analysis data to $file\n"
      if $opt{verbose};

    my $rdb = RDB->new( $file, '>' )
      or die( "unable to create $file\n" );

    $rdb->init(
        tmin => 'N',
        tmax => 'N',
        mask => 'N',
        map { ( "x_$_" => 'N', "y_$_" => 'N' ) }
          qw( n ave cent fq lq median rms )
    );

    my %data;
    for my $bin ( 0 .. $tmin->nelem - 1 ) {
        $data{tmin} = $tmin->at( $bin );
        $data{tmax} = $tmax->at( $bin );
        $data{mask} = $mask->at( $bin );

        foreach my $coord ( qw( x y ) ) {
            $data{"${coord}_$_"} = $data->{$coord}{$_}->at( $bin )
              foreach qw( n ave cent fq lq median rms );
        }
        $rdb->write( \%data );
    }
}


sub center_data {
    my ( $x, $y, $xc, $yc, $verbose ) = @_;

    my $cent;

    unless ( $opt{force_ctr} ) {
        print "Centering data\n"
          if $verbose;

        $cent = sigma_clip(
            coords     => [ $x,  $y ],
            center     => [ $xc, $yc ],
            iterlim    => $opt{iter},
            log        => $verbose,
            clip       => $opt{iclip},
            nsigma     => $opt{nsigma},
            dtol       => $opt{dtol},
            log        => $verbose,
        );

    }

    else {
        my $dist2 = ( $x - $opt{xc} )**2 + ( $y - $opt{yc} )**2;

        $cent = {
            error => 0,
            center => pdl( $opt{xc}, $opt{yc} ),
            clip   => $opt{clip_r},
            nelem  => ( $dist2 < $opt{clip_r}**2 )->sum,
        };

    }

    print Data::Dumper->Dump( [$cent], ['Center'] )
      if $verbose > 3;

    return $cent;
}


sub parse_args {
    Getopt::ArgvFile::argvFile;

    %opt = (
        # input
        input   => undef,
        extname => 'events',
        xcol    => 'x',
        ycol    => 'y',
        tcol    => 'time',

        scale => 1,
        unit  => 'pixel',

        clip_sigma => 2,

        # output
        tag     => undef,
        device  => '/xs',
        xsid    => 5,
        title   => 1,

        # center
        dtol	=> 0.01,
        force_ctr => 0,
        iter      => 20,
        nsigma    => 3,
    );


    eval {
        local $SIG{__WARN__} = sub { die $_[0] };

        GetOptions(
            \%opt,
            qw{
              tag=s
              input=s
              extname=s
              filter=s
              gti=s
              xcol=s
              ycol=s
              tcol=s
              scale=f
              unit=s

              dtol=f
              force_ctr
              iclip=f
              iter=i
              nsigma=f
              pixcent:i
              xc=f
              yc=f

              nbins=i
              binw=f
              clip_sigma=f
              clip_r=f

              hist_d_width=f
              hist_r_width=f
              hist_p_width=f
              hist_d_lims=s
              hist_r_lims=s
              hist_p_lims=s

              device=s
              xsid=i
              title!
              width=f

              verbose:i
              help
              usage
              version
              } );

    };

    die $@ if $@;

    return if $opt{help} || $opt{usage} || $opt{version};

    if ( defined $opt{verbose} ) {
        $opt{verbose} ||= 1;
    }

    else {
        $opt{verbose} = 0;
    }

    $opt{input} = shift @ARGV if @ARGV && !defined $opt{input};

    my @notset = grep { !defined $opt{$_} } keys %opt;
    die( 'must set these option(s): ', join( ', ', @notset ), "\n" )
      if @notset;

    die( "Extra arguments on the command line: @ARGV\n" )
      if @ARGV;

    die( "must specify --clip_r if --force_ctr is specified\n" )
      if ( $opt{force_ctr} && !exists $opt{clip_r} );

    die( "must specify only one of --nbins or --binw\n" )
      if ( exists $opt{nbins} && exists $opt{binw} );

    $opt{pixcent} = 256
      if defined $opt{pixcent} && 0 == $opt{pixcent};

    $opt{nbins} = 100 unless exists $opt{nbins} || $opt{binw};

    $opt{iclip} ||= 1000 * $opt{scale};

    $opt{unit} = '[' . $opt{unit} . ']';

    $PlotOpts{WindowWidth} = $opt{width} if exists $opt{width};

    for my $lim ( qw( d r p ) ) {
        my $flim = "hist_${lim}_lims";
        if ( exists $opt{$flim} ) {
            $opt{$flim} = [ split( ',', $opt{$flim}, 4 ) ];
        }
        else {
            $opt{$flim} = [ undef, undef, undef, undef ];
        }
    }
    $opt{xc} = undef unless exists $opt{xc};
    $opt{yc} = undef unless exists $opt{yc};

}


sub help {
    my ( $verbose ) = @_;

    require Pod::Usage;
    Pod::Usage::pod2usage( { -exitval => 0, -verbose => $verbose } );
}

__END__

=pod

=head1 NAME

analycent - plot diagnostics of image centroid motion

=head1 SYNOPSIS

B<analycent> I<options> [F<input file>]

=head1 OPTIONS

B<analycent> uses long options. Options may be abbreviated, and the
"=" character shown below in the option templates is optional.  A
command line argument which begins with the C<@> character is taken to
be the name of a file which contains white-space delimited options.

=head2 Input Data Specification

=over 8

=item --input=F<file>

The input events FITS file.  The file may also be specified as the
last argument on the command line.

=item --extname=I<string>

The FITS extension name of the HDU to be read from the input file.
It defaults to C<events>

=item --filter=I<string>

A CFITSIO compatible events filter

=item --xcol=I<string>
 --ycol=I<string>
 --tcol=I<string>

The names of the event position and time columns to use.  These default to
C<x>, C<y>, and C<time>, respectively.

=item --gti=I<file>

A CFITSIO compatible file/HDU specification of a good time interval
list to apply to the data.

=item --scale=I<float>

A factor by which the input event positions will be multiplied (for units
conversion).  It defaults to C<1>.

=item --units=I<string>

The position units (after scaling by B<--scale>). For plotting purposes.
Defaults to C<pixels>.

=back

=head2 Output Options

=over 8

=item --tag=I<string>

The prefix used for the output files.  This option is not optional.

=item --device=I<PGPLOT device>

The output plotting device.  It defaults to C</xs>, which plots
to the screen.  Useful other ones are C</cps> and C</vcps>, which
are, respectively, landscape and portrait color PostScript output.

Hardcopy will be left in files with extensions of F<.ps> for PostScript,
F<.png> for PNG, etc.

=item --xsid

When plotting to the C</xs> device, multiple windows are used, one per plot.
The starting PGPLOT window id may be specified with this option.  It defaults
to C<5>.

=item --title, --notitle

Do (don't) write a title on the plots.  Default is to write the title.

=item --width=I<float>

Specify the (approximate) width of the resultant plots, in inches.

=item --verbose, --verbose=I<level>

Be noisy. A verbosity level > 1 will also output progress in reading the input file.

=back

=head2 Image Centering Options

=over 8

=item --force_ctr

If specified, the point specified by the B<--xc> and B<--yc> options will
be used as the center, rather than being determined by an iterative
sigma clipping method.  The B<--clip_r> option must be set if
B<--force_ctr> is used.

=item --dtol=I<float>

Absolute convergence tolerance for the iterative centering algorithm.
It defaults to C<0.01>.

=item --iclip=I<float>

The initial centering clipping radius, in scaled position units.  It
defaults to C<1000>.

=item --iter=I<integer>

The number of iterations the centering routine should perform.  It
defaults to C<10>.

=item --nsigma=I<float>

The clipping radius, in units of the standard deviation of the event
distribution about the center, used during the iterative centering. It
defaults to C<3>.

=item --xc=I<float>

If set, this specifies an initial value (in scaled units) for the I<X>
coordinate of the center of the object.  See B<--scale>.

=item --yc=I<float>

If set, this specifies an initial value (in scaled units) for the I<Y>
coordinate of the center of the object.  See B<--scale>.

=item --pixcent

If specified, choose the brightest pixel in the image as the initial
center.  This option optionally takes an integer argument specifying
the image size in pixels (the image is square).  This defaults to 256.


=back

=head2 Analysis Options

=over 8

=item --nbins=I<integer>

The number of time bins into which to divide the exposure.  This
defaults to C<100>.  See also B<--binw>.

=item --binw=I<float>

The time bin width, in seconds. See also B<--nbins>.

=item --clip_sigma=I<float>

The radius of the circular region around the object used for the analysis,
in units of the calculated standard deviation of the event distribution
about the center.  See also B<--clip_r>.  Defaults to C<2>.

=item --clip_r=I<float>

The radius of the circular region around the object used for the analysis,
in scaled units.  See alsc B<--clip_sigma>.  This must be specified
if B<--force_ctr> is used.

=item --hist_d_width=I<float>

The width of the bins used to generate the histogram of average
distances to the center.  By default 25 bins are created.

=item --hist_r_width=I<float>

The width of the bins used to generate the RMS histogram.
By default 25 bins are created.

=item --hist_p_width=I<float>

The width of the bins used to generate the first - last quartile histogram.
By default 25 bins are created.

=item --hist_d_lim=I<xmin,xmax,ymin,ymax>

Plot limits for the distance histogram. Values not specified
will be determined from the data.

=item --hist_r_lim=I<xmin,xmax,ymin,ymax>

Plot limits for the RMS histogram. Values not specified
will be determined from the data.

=item --hist_p_lim=I<xmin,xmax,ymin,ymax>

Plot limits for the first - last quartile histogram. Values not specified
will be determined from the data.


=back

=head2 Miscellaneous Options

=over 8

=item --help

Output abbreviated usage information and exit.

=item --usage

Output extensive usage information and exit.

=item --version

Output version information and exit.

=back


=head1 DESCRIPTION

B<analycent> produces some simple diagnostics of the motion of the
centroid of a point source as a function of time.  It divides
an observation into a number of time bins and calculates the
events statistics in each bin in a circular region centered
about the center of the point source as determined in that bin.
(Bins with outrageously large clipping regions (greater than 3 sigma
from and iteratively determiend averaged) are not plotted, but
are written to the output RDB table).

The input events must be in a binary table extension of a FITS file.
CFITSIO is used to extract the data, so the CFITSIO extended filename
syntax may be used to filter the data. The event positions may be
scaled (to convert to seconds of arc, for example).

The center of the circular region may be specified by the user, via
the B<--xc>, B<--yc> and B<--force_ctr> options, or may be determined
from the data by centering using a sigma clipping algorithm.  In the
latter case, an initial position may be specified using B<--xc> and
B<--yc>.  The options for the centering algorithm may be modified
with the B<--dtol>, B<--iclip>, B<--iter>, and B<--nsigma>.

The radius of the analysis region is determined either from the
standard deviation of the event distribution about the center, if
B<analycent> determined the center and B<--clip_sigma> is specified,
or may be given directly via the B<--clip_r> option.

=head2 Output

=head3 Plots

In the following descriptions, I<.sfx> refers to the filename suffix
appropriate to the hard copy format (e.g, I<.ps>, I<.png>)

=over 8

=item Event Positions

The I<X> and I<Y> event positions relative to the object's center are
plotted.  Hardcopy will be left in the file F<tag_events.sfx>.

=item Time variation of image centroids

The average offset of events from the object's center are
determined for each time slice in the circular region and are plotted
as a function of time.  Hardcopy will be left in a file
F<tag_pts_dist.sfx>.

A histogram of the offsets is also provided.
Hardcopy will be left in the file F<tag_hist_dist.sfx>

The 2D motion of the events relative to the image centroid is also plotted.
Hardcopy will be left in the file F<tag_pts_motion.sfx>.

=item Time variation of image width.

=over 8

=item *

The RMS of events' positions is determined.

A scatter plot (position vs. time) is sent to  I<tag>_pts_rmsI<.sfx>.
A histogram is sent to F<tag_hist_rms.sfx>.

=item *

The difference between the first and last quartile positions is plotted
against time in F<tag_pts_mq.sfx>.  A histogram is plotted to
F<tag_hist_mq.sfx>

=back

=back

=head3 Output data

An RDB table is written to F<tag_stats.rdb> with the following columns,
one per time bin:

=over 8

=item tmin, tmax

The bin limits.

=item mask

True if the bin was not outrageously out of line, based on the
radius of the clipping region used to select events in the time bin.

=item x_ave, y_ave

The average of the X and Y event positions.

=item x_cent, y_cent

The iteratively determined center of the positions in the bin

=item x_fq, y_fq, x_lq, y_lq

The first and last quartiles of the data

=item x_median, y_median

The median of the data

=item x_rms, y_rms

The RMS of the data

=back

=head1 VERSION

$Revision$

=head1 AUTHOR

Diab Jerius (djerius@cfa.harvard.edu)

=cut
